indexing
	description: "GUI Test Driver"
	author: "Peter Brandt and Martin Geisler"
	date: "$Date: 2005-06-26 21:26:20 +0200 (Sun, 26 Jun 2005) $"
	revision: "$Revision: 107 $"

class
	GRAPHICAL_TEST_DRIVER

inherit
	TEST_DRIVER
		rename
			execute as execute_tests
		undefine
			default_create,
			copy
		redefine
			make
		end
	
	EV_TITLED_WINDOW
		rename
			raise as w_raise
		undefine
			count
		redefine
			initialize
		end
	
	INTERFACE_SHARED
		undefine
			default_create,
			copy
		end
	
	EXCEP_CONST
		undefine
			default_create,
			copy
		end

create
	make

feature {NONE} -- Initialization

	make is
			-- Create the test case and add GUI specific actions to the
			-- execute_started_actions from TEST_DRIVER
		do
			Precursor
			execute_started_actions.extend (agent reset_widgets)
			execute_started_actions.extend (agent execute_started_action)
			execute_finished_actions.extend (agent execute_finished_action)
			
			create application
			default_create
		end

	initialize is
			-- Initialize `Current'.
		do
			Precursor {EV_TITLED_WINDOW}

			create main_container
			main_container.set_border_width (Widget_margin)
			main_container.set_padding_width (Widget_margin)
			put (main_container)

			build_progress_frame
			build_detail_frame
			reset_widgets
			-- Disable until the tests have been run
			detail_frame.disable_sensitive

			-- Tear down entire application on close
			close_request_actions.extend (agent destroy_application)
			
			test_executed_actions.extend (agent record_progress)
			start_button.select_actions.extend (agent execute_tests)
			
			previous_button.select_actions.extend (agent previous_test_action)
			next_button.select_actions.extend (agent next_test_action)
			detail_button.select_actions.extend (agent detail_button_action)
			
			set_width (Window_width)
			set_height (Window_height)
			set_title (Graphical_test_driver_window_title)
			show
		end
	
	build_progress_frame is
			-- Populate `progress_frame'
		require
			progress_frame_void: progress_frame = Void
		local
			progress_hbox: EV_HORIZONTAL_BOX
			progress_vbox: EV_VERTICAL_BOX
			progress_box: EV_VERTICAL_BOX
		do
			create progress_frame.make_with_text (Progress_frame_text)
			main_container.extend (progress_frame)

			create progress_box
			progress_box.set_border_width (Widget_margin)
			progress_box.set_padding_width (Widget_margin)
			progress_frame.put (progress_box)
			
			create progress_hbox
			progress_hbox.set_padding_width (Widget_margin)
			progress_box.extend (progress_hbox)
			
			create progress_vbox
			progress_vbox.set_padding_width (Widget_margin)
			progress_hbox.extend (progress_vbox)
			
			create passed_label
			progress_vbox.extend (left_align (passed_label))

			create failed_label
			progress_vbox.extend (left_align (failed_label))

			create total_label
			progress_vbox.extend (left_align (total_label))
			
			create start_button.make_with_text (Start_button_text)
			progress_hbox.extend (start_button)

			create progress_bar
			progress_box.extend (progress_bar)			
		ensure
			progress_frame_not_void: progress_frame /= Void
		end

	build_detail_frame is
			-- Populate `detail_frame'
		require
			detail_frame_void: detail_frame = Void
		local
			detail_box: EV_VERTICAL_BOX
			detail_hbox: EV_HORIZONTAL_BOX
		do
			create detail_frame.make_with_text (Detail_frame_text)
			main_container.extend (detail_frame)

			create detail_box
			detail_box.set_border_width (Widget_margin)
			detail_box.set_padding_width (Widget_margin)
			detail_frame.put (detail_box)
			
			create test_name_label
			create status_label
			create class_label
			create routine_label
			create tag_label
			create violation_type_label

			detail_box.extend (left_align (test_name_label))
			detail_box.extend (left_align (status_label))
			detail_box.extend (left_align (class_label))
			detail_box.extend (left_align (routine_label))
			detail_box.extend (left_align (tag_label))
			detail_box.extend (left_align (violation_type_label))
			
			create previous_button.make_with_text (Previous_button_text)
			create next_button.make_with_text (Next_button_text)
			create detail_button.make_with_text (Detail_button_text)

			create detail_hbox
			detail_hbox.set_padding_width (Widget_margin)
			detail_hbox.extend (previous_button)
			detail_hbox.extend (next_button)
			detail_hbox.extend (detail_button)
			
			detail_box.extend (detail_hbox)
		ensure
			detail_frame_not_void: detail_frame /= Void
		end
		
	reset_widgets is
			-- Reset labels to default state
		do
			passed_label.set_text (Passed_label_text + "0")
			failed_label.set_text (Failed_label_text + "0")
			total_label.set_text (Total_label_text + "0")

			progress_bar.value_range.resize_exactly (0, count)
			progress_bar.set_value (0)
			
			test_name_label.set_text (Test_name_label_text)
			status_label.set_text (Status_label_text)
			class_label.set_text (Class_label_text)
			routine_label.set_text (Routine_label_text)
			tag_label.set_text (Tag_label_text)
			violation_type_label.set_text (Violation_type_label_text)
		end

feature -- Basic operations

	execute is
			-- Launch application
		do
			application.launch
		end

feature {NONE} -- Implementation

	execute_started_action is
			-- Prepare for execution
		do
			-- Disable the button and the detail frame while the tests are running
			detail_frame.disable_sensitive
			start_button.disable_sensitive
		end
	
	execute_finished_action is
			-- Wrap up execution
		do
			detail_frame.enable_sensitive
			-- Rewind and show details of first test case
			test_cases.start
			update_detail_widgets(test_cases.item)	
		end

	record_progress (test: TEST_CASE) is
			-- Updates progress label
		do
			update_progress_widgets (test)
			update_detail_widgets (test)
			-- Allow the window to react to the changed widgets
			(create {EV_ENVIRONMENT}).application.process_events
		end

	update_progress_widgets(test: TEST_CASE) is
			-- Fill in widgets in the Progress frame
		require
			test_executed: test /= Void and then test.has_executed
		do
			progress_bar.step_forward

			passed_label.set_text (Passed_label_text + passed.out)
			failed_label.set_text (Failed_label_text + failed.out)
			total_label.set_text (Total_label_text + (passed + failed).out)
		ensure
			progress_bar_stepped:
				progress_bar.value = old progress_bar.value + progress_bar.step
			passed_label_set:
				passed_label.text.is_equal (Passed_label_text + passed.out)
			failed_label_set:
				failed_label.text.is_equal (Failed_label_text + failed.out)
			total_label_set:
				total_label.text.is_equal (Total_label_text + (passed + failed).out)
		end
		
	update_detail_widgets (test: TEST_CASE) is
			-- Fill in widgets in the Test Detail frame
		require
			test_executed: test /= Void and then test.has_executed
		do
			test_name_label.set_text (Test_name_label_text + test.name)

			if test.has_failed then
				status_label.set_text (Status_label_text + Failed_text)
				class_label.set_text (Class_label_text + test.class_name)
				routine_label.set_text (Routine_label_text + test.routine_name)
				tag_label.set_text (Tag_label_text + test.tag_name)
				update_violation_label (test) -- More complex so it has it's own routine
			else
				status_label.set_text (Status_label_text + Passed_text)
				class_label.set_text (Class_label_text + None_text)			
				routine_label.set_text (Routine_label_text + None_text)
				tag_label.set_text(Tag_label_text + None_text)
				violation_type_label.set_text (Violation_type_label_text + None_text)
			end

			if test_cases.isfirst then
				previous_button.disable_sensitive
			else
				previous_button.enable_sensitive
			end

			if test_cases.islast then
				next_button.disable_sensitive
			else
				next_button.enable_sensitive
			end

			if test.saved_states.is_empty then
				detail_button.disable_sensitive
			else
				detail_button.enable_sensitive
			end
		ensure
			class_label_set:
				test.has_failed implies class_label.text.is_equal (Class_label_text + test.class_name)
			routine_label_set:
				test.has_failed implies routine_label.text.is_equal (Routine_label_text + test.routine_name)
			tag_label_set:
				test.has_failed implies tag_label.text.is_equal (Tag_label_text + test.tag_name)
		end
	
	update_violation_label (test: TEST_CASE) is
			-- Update `violation_type_label' based on `test'
		require
			test_failed: test /= Void and then test.has_failed
		local
			text: STRING
		do
			inspect test.exception
			when Void_call_target then
				text := Void_call_target_text
			when No_more_memory then
				text := No_more_memory_text
			when Precondition then
				text := Precondition_text
			when Postcondition then
				text := Postcondition_text
			when Floating_point_exception then
				text := Floating_point_exception_text
			when Class_invariant then
				text := Class_invariant_text
			when Check_instruction then
				text := Check_instruction_text
			when Routine_failure then
				text := Routine_failure_text
			when Incorrect_inspect_value then
				text := Incorrect_inspect_value_text
			when Loop_variant then
				text := Loop_variant_text
			when Loop_invariant then
				text := Loop_invariant_text
			when Signal_exception then
				text := Signal_exception_text
			when Rescue_exception then
				text := Rescue_exception_text
			when External_exception then
				text := External_exception_text
			when Void_assigned_to_expanded then
				text := Void_assigned_to_expanded_text
			when Io_exception then
				text := Io_exception_text
			when Operating_system_exception then
				text := Operating_system_exception_text
			when Retrieve_exception then
				text := Retrieve_exception_text
			when Developer_exception then
				text := Developer_exception_text
			when Runtime_io_exception then
				text := Runtime_io_exception_text
			when Com_exception then
				text := Com_exception_text
			when Runtime_check_exception then
				text := Runtime_check_exception_text
			else
				text := Unknown_exception_text + test.exception.out
			end
			
			violation_type_label.set_text (Violation_type_label_text + text)
		end
		

	next_test_action is
			-- Move to next test, update detail widgets
		require
			next_exixts: not test_cases.islast
		do
			test_cases.forth
			update_detail_widgets (test_cases.item)
		end
	
	previous_test_action is
			-- Move to previous test, update detail widgets
		require
			previous_exists: not test_cases.isfirst
		do
			test_cases.back
			update_detail_widgets (test_cases.item)
		end
		
	detail_button_action is
			-- Show detail for saved state associated with current test
		require
			has_saved_states: test_cases.item.has_executed and then
				not test_cases.item.saved_states.is_empty
		local
			window: SAVED_STATE_INSPECTOR
		do
			create window.make(test_cases.item.saved_states)
			window.show			
		end

	destroy_application is
			-- Terminate the application
		do
			-- Terminate the application
			(create {EV_ENVIRONMENT}).application.destroy
		end

feature {NONE} -- Implementation / Widgets

	application: EV_APPLICATION
	
	main_container: EV_VERTICAL_BOX

	-- Widgets in the Overall Progress frame
	progress_frame: EV_FRAME
	progress_bar: EV_HORIZONTAL_PROGRESS_BAR
	passed_label: EV_LABEL
	failed_label: EV_LABEL
	total_label: EV_LABEL
	start_button: EV_BUTTON
	
	-- Widgets in the Test Detail frame
	detail_frame: EV_FRAME
	test_name_label: EV_LABEL
	status_label: EV_LABEL
	class_label: EV_LABEL
	routine_label: EV_LABEL
	tag_label: EV_LABEL
	violation_type_label: EV_LABEL
	previous_button: EV_BUTTON
	next_button: EV_BUTTON
	detail_button: EV_BUTTON

		

feature {NONE} -- Implementation / Constants

	Window_width: INTEGER is 0
			-- Initial window width
			--| Zero pixels make the image was narrow as possible
	Window_height: INTEGER is 0
			-- Initial window height
			--| Zero pixels make the image as short as possible

end -- class GRAPHICAL_TEST_DRIVER
