indexing
	description: "Saved states capture the state of an execution"
	author: "Peter Brandt and Martin Geisler"
	date: "$Date: 2005-06-25 14:42:10 +0200 (Sat, 25 Jun 2005) $"
	revision: "$Revision: 100 $"

class
	SAVED_STATE

create
	make

feature {NONE} -- Initialization

	make (a_current_object: ANY; a_routine_name: STRING; some_routine_arguments: TUPLE) is
			-- Initialize `Current' with deep twins of the arguments
			-- `a_current_object' and `some_routine_arguments'
		require
			current_object_not_void: a_current_object /= Void
			routine_name_not_void: a_routine_name /= Void
			routine_arguments_not_void: some_routine_arguments /= Void
		do
			-- Careful! `a_current_object' might violate its class
			-- invariant, and so we cannot rely on
			-- `a_current_object.deep_twin' --- if the class invariant
			-- is broken, then any feature call will generate an
			-- exception. But `deep_twin' actually calls
			-- `c.deep_clone', so we can do the same:
			current_object := feature {ISE_RUNTIME}.c_deep_clone ($a_current_object)

			-- Assumed to be a manifest constant so just do a normal
			-- reference assignment:
			routine_name := a_routine_name

			-- We consider a `TUPLE' safe (and always obey its class
			-- invariant), so just call `deep_twin' normally:
			routine_arguments := some_routine_arguments.deep_twin
		ensure
			current_object_cloned: deep_equal (current_object, a_current_object)
			routine_name_set: routine_name = a_routine_name
			routine_arguments_cloned: deep_equal (routine_arguments, some_routine_arguments)			
		end

feature -- Access

	current_object: ANY
	routine_name: STRING
	routine_arguments: TUPLE

invariant
	current_object_not_void: current_object /= Void
	routine_name_not_void: routine_name /= Void
	routine_arguments_not_void: routine_arguments /= Void

end -- class SAVED_STATE
