indexing
	description: "Manager of Saved States"
	author: "Peter Brandt and Martin Geisler"
	date: "$Date: 2005-06-28 00:06:33 +0200 (Tue, 28 Jun 2005) $"
	revision: "$Revision: 115 $"

class
	STATE_MANAGER

feature -- Access

	save_state (current_object: ANY; routine_name: STRING; routine_arguments: TUPLE) is
			-- Save arguments as a new `SAVED_STATE' object
		require
			current_object_not_void: current_object /= Void
			routine_name_not_void: routine_name /= Void
			routine_arguments_not_void: routine_arguments /= Void
		local
			state: SAVED_STATE
		do
			create state.make(current_object, routine_name, routine_arguments)
			Saved_states.extend (state)
		ensure
			one_more: Saved_states.count = old Saved_states.count + 1
		end

feature {TEST_CASE} -- Access for `TEST_CASE'

	wipe_out_saved_states is
			-- Wipe out `Saved_states'
		do
			Saved_states.wipe_out
		ensure
			wiped_out: Saved_states.is_empty
		end
	
	saved_states_twin: LIST [SAVED_STATE] is
			-- A twin of `Saved_states'
		do
			Result := Saved_states.twin
		ensure
			equal: Result.is_equal (Saved_states)
		end

feature {NONE}

	Saved_states: LIST [SAVED_STATE] is
			-- Saved states, shared between all state managers
		once
			create {LINKED_LIST [SAVED_STATE]} Result.make
		end

end -- class STATE_MANAGER
