indexing
	description: "Manager of Test Cases"
	author: "Peter Brandt and Martin Geisler"
	date: "$Date: 2005-06-28 00:06:33 +0200 (Tue, 28 Jun 2005) $"
	revision: "$Revision: 115 $"

class
	TEST_DRIVER
	
inherit
	EXCEPTIONS

create
	make
	
feature {NONE} -- Initialization

	make is
			-- Creation procedure
		do
			create {LINKED_LIST [TEST_CASE]} test_cases.make
			create test_executed_actions
			create execute_started_actions
			create execute_finished_actions
		end
		
feature -- Measurement

	count: INTEGER is
			-- Number of test cases
		do
			Result := test_cases.count
		end
	
	passed: INTEGER
			-- Number of succesfully executed test cases 

	failed: INTEGER
			-- Number of failed test cases

feature -- Events

	execute_started_actions: ACTION_SEQUENCE [TUPLE []]
		-- Actions to call when `execute_tests' is called

	execute_finished_actions: ACTION_SEQUENCE [TUPLE []]
		-- Actions to call when `execute_tests' is finished

	test_executed_actions: ACTION_SEQUENCE [TUPLE [TEST_CASE]]
		-- Actions which will be triggered after each executed test case

feature -- Element change
	add_test_case (test: TEST_CASE) is
			-- Add a test case
		require
			test_not_void: test /= Void
		do
			test_cases.extend (test)
		ensure
			test_added: test_cases.last = test
			one_more: test_cases.count = old test_cases.count + 1
		end

feature -- Basic operations

	execute is
			-- Execute all test cases
		do
			passed := 0
			failed := 0
			execute_started_actions.call ([])
			from
				test_cases.start
			until
				test_cases.after
			loop
				test_cases.item.execute
			
				if test_cases.item.has_failed then
					failed := failed + 1
				else
					passed := passed + 1
				end
				-- Trigger any listeners
				test_executed_actions.call ([test_cases.item])
				test_cases.forth
			end
			execute_finished_actions.call ([])
		ensure
			all_executed: failed + passed = count
		end

feature {NONE} -- Implementation
	test_cases: LIST [TEST_CASE]
		-- Test cases

invariant
	test_cases_not_void: test_cases /= Void
	test_executed_actions_not_void: test_executed_actions /= Void
	execute_started_actions_not_void: execute_started_actions /= Void
	execute_finished_actions_not_void: execute_finished_actions /= Void
	passed_non_negative: passed >= 0
	failed_non_negative: failed >= 0
	consistant_sum: passed + failed <= count

end -- class TEST_DRIVER
